@@ -8,8 +8,9 @@ import sys
 
 import requests
 import yaml
+import time
 
-VERSION = '0.1.1'
+VERSION = '0.1.2'
 
 default_backliftURL = 'http://backlift.com/'
 backlift_configfile = '.backlift'
@@ -18,74 +19,135 @@ backlift_email = 'support@backlift.com'
 # maxfiles is set to a few less than the system limit. 
 maxfiles = resource.getrlimit(resource.RLIMIT_NOFILE)[0] - 6
 
+## ERROR MESSAGES -------------------------------------------------------------
+
+ERR_ID =       "Ooops! Something's wrong. I couldn't obtain an app id. To\n"+\
+               "use the push or watch commands, you need an existing\n"+\
+               "backlift app created with either the create or init commands."
+
+ERR_BADID =    "Ooops! Something's wrong. I couldn't obtain an app id from\n"+\
+               "the server. Please send an angry email to %s." % backlift_email
+
+ERR_MAXFILE =  "Too many files. Total: %d,  max: %d"
+
+ERR_SERVER =   "Ooops! Our server is not responding. Hopefully this is\n"+\
+               "temporary. If this problem persists, please send an angry\n"+\
+               "email to %s." % backlift_email
+
+ERR_WRITE =    "Ooops! We couldn't create a file. Please check to ensure\n"+\
+               "you have permission to write to %s."
+
+ERR_INIT =     "This app has already been initialized."
+
+ERR_500 =      "Ooops! Our server is having trouble. We're looking into\n"+\
+               "it! While we're at it, please check to make sure you're\n"+\
+               "running the latest version of the backlift command line\n"+\
+               "interface at www.backlift.com. You're currently running\n"+\
+               "backlift cli %s." % VERSION
+
+ERR_404 =      "Ooops! We couldn't find the resource at %s."
+
+ERR_403 =      "Ooops! This action is forbidden. Either you haven't\n" +\
+               "registered your early access code, or you used an\n"+\
+               "invalid code. Please see backlift --help."
+
+ERR_RESPONSE = "Oops! Something is wrong with the server. Please send an\n"+\
+               "angry email to %s." % backlift_email
+
+
 ## UTILITIES ------------------------------------------------------------------
 
-def collectFiles(path, skip_hidden):
-    files = {}        
-    count = 0
-    configfile = ''
+def fail(msg):
+    print >> sys.stderr, msg
+    sys.exit(1)
 
-    rootpath = os.path.normpath(os.path.abspath(path))
-    print "Scanning %s" % rootpath
-
-    for (dirpath, dirnames, filenames) in os.walk(rootpath):
-        # import pdb; pdb.set_trace()
-        hidden = reduce(lambda x, y: x or (y[0] == '.'), 
-            dirpath.split('/')[1:], False)
-
-        if not hidden or not skip_hidden:
-
-            for name in filenames:
-
-                filepath = os.path.normpath(os.path.abspath(
-                    os.path.join(dirpath, name)))
-
-                if name == backlift_configfile and dirpath == rootpath:
-                    configfile = filepath
-                elif skip_hidden and name[0] == '.':
-                    continue
-
-                count += 1
-
-                if count > maxfiles:
-                    continue
-
-                if os.path.isfile(filepath):
-                    filekey = filepath.replace(rootpath, '', 1).strip('/')
-                    files[filekey] = (filekey, open(filepath, 'rb'))
-                    print "Adding %s" % filekey
-
-    return files, count, configfile
 
 def check_response(r):
     if not (r.status_code / 100 == 2):
         if r.text:
             mtype = r.headers['Content-Type'].split(';')[0]
             if r.status_code == 500:
-                print >> sys.stderr, \
-                    "Ooops! Our server is having trouble. We're looking "+\
-                    "into it!\nWhile we're at it, please check to make sure "+\
-                    "you're running\nthe latest version of the backlift "+\
-                    "command line interface at\nwww.backlift.com. You're "+\
-                    "currently running backlift cli "+VERSION
+                fail(ERR_500)
             elif r.status_code == 404:
-                print >> sys.stderr, \
-                    "Ooops! We couldn't find the resource at " +\
-                    "%s.\n" % r.request.url + "To reset this app's URLs, " +\
-                    "delete the .backlift file in your app's root folder."
+                fail(ERR_404 % r.request.url)
             elif r.status_code == 403:
-                print >> sys.stderr, \
-                    "Ooops! This action is forbidden. Either you haven't \n" +\
-                    "registered your early access code, or you used an \n"+\
-                    "invalid code. Please see backlift --help."
+                fail(ERR_403)
             else:
-                print >> sys.stderr, r.text
+                fail(ERR_RESPONSE)
         else: 
-            r.raise_for_status()
+            fail(ERR_RESPONSE)
+
+
+def listFiles(path, skip_hidden):
+    files = []        
+    configfile = ''
+
+    for (dirpath, dirnames, filenames) in os.walk(path):
+        # import pdb; pdb.set_trace()
+        path_hidden = reduce(lambda x, y: x or (y[0] == '.'), 
+            dirpath.split('/')[1:], False)
+
+        if not path_hidden or not skip_hidden:
+
+            for name in filenames:
+
+                filepath = os.path.normpath(os.path.join(dirpath, name))
+
+                if name == backlift_configfile and dirpath == path:
+                    configfile = filepath
+                elif skip_hidden and name[0] == '.':
+                    continue
+
+                files.append(filepath)
+
+    filecount = len(files)
+    if filecount > maxfiles:
+        fail(ERR_MAXFILE % (filecount, maxfiles))
+
+    return files, configfile
+
+
+def collectFiles(path, skip_hidden):
+    # import pdb; pdb.set_trace()
+    rootpath = os.path.normpath(os.path.abspath(path))
+    print "Scanning %s" % rootpath
+
+    filelist, configfile = listFiles(rootpath, skip_hidden)
+
+    files = {}
+
+    for filepath in filelist:
+        filekey = filepath.replace(rootpath, '', 1).strip('/')
+        files[filekey] = (filekey, open(filepath, 'rb'))
+        print "Adding %s" % filekey
+
+    return files, configfile
+
+
+def scanFiles(path, skip_hidden):
+    rootpath = os.path.normpath(os.path.abspath(path))
+    filelist, configfile = listFiles(rootpath, skip_hidden)
+    return dict ([(f, os.stat(f).st_mtime) for f in filelist]),\
+           configfile
+
+
+def get_id(configfile):
+    id = ''
+    try:
+        handle = open(configfile, 'r')
+        cfg_str = handle.read()
+        cfg_data = yaml.safe_load(cfg_str)
+        handle.close()
+        id = cfg_data['_app_id']
+    except Exception as e:
+        pass
+
+    if not id: fail(ERR_ID)
+
+    return id
+
 
-        sys.exit(1)
 
-    return
 
 
 ## REQUESTS -------------------------------------------------------------------
@@ -96,9 +158,8 @@ def register_key(key):
         r = requests.post(backliftURL_registerkey, params={'access_key': key})
         check_response(r)
     except requests.exceptions.ConnectionError:
-        print >> sys.stderr, "Ooops! Our server is not responding. " +\
-            "Hopefully this is temporary. \nIf this problem persists, " +\
-            "please send an angry email to %s." % backlift_email
+        fail(ERR_SERVER)
+
 
 def create_app():
     try:
@@ -110,12 +171,11 @@ def create_app():
         return r['_id']
 
     except requests.exceptions.ConnectionError:
-        print >> sys.stderr, "Ooops! Our server is not responding. " +\
-            "Hopefully this is temporary. \nIf this problem persists, " +\
-            "please send an angry email to %s." % backlift_email
+        fail(ERR_SERVER)
 
     return None
 
+
 def download_template_files(path, template, params=None):
     files = []
     try:
@@ -148,19 +208,14 @@ def download_template_files(path, template, params=None):
                         os.makedirs(filedir, 0755)
                         newfile = open(filepath, 'wb')
                     except OSError:
-                        print >> sys.stderr, "Ooops! We couldn't create "+\
-                            "the app template. Please check to ensure "+\
-                            "you have permission to write to %s." % filedir
-                        return
+                        fail(ERR_WRITE % filedir)
 
                 newfile.write(r.content)
                 newfile.close()
                 print "creating %s" % os.path.join(path, f)
 
     except requests.exceptions.ConnectionError:
-        print >> sys.stderr, "Ooops! Our server is not responding. " +\
-            "Hopefully this is temporary. \nIf this problem persists, " +\
-            "please send an angry email to %s." % backlift_email
+        fail(ERR_SERVER)
 
     return None
 
@@ -181,9 +236,7 @@ def upload_files(id, files):
         print "Your app is hosted at -->> \n%s\n" % r['app_url']
 
     except requests.exceptions.ConnectionError:
-        print >> sys.stderr, "Ooops! Our server is not responding. " +\
-            "Hopefully this is temporary. \nIf this problem persists, " +\
-            "please send an angry email to %s." % backlift_email
+        fail(ERR_SERVER)
 
 
 ## COMMANDS -------------------------------------------------------------------
@@ -191,6 +244,7 @@ def upload_files(id, files):
 def register(key):
     register_key(key)
 
+
 def create(path, name='', template='blank'):
 
     # import pdb; pdb.set_trace()
@@ -202,20 +256,16 @@ def create(path, name='', template='blank'):
     cfg_path = os.path.join(app_root, backlift_configfile)
 
     if (os.path.exists(cfg_path)):
-        print >> sys.stderr, "This app has already been initialized."
-        return
-
+        fail(ERR_INIT)
+ 
     id = create_app()
     if id:
         download_template_files(app_root, template, {
             'app_id': id
         })
 
-    if not id:
-        print >> sys.stderr, "Ooops! Something's wrong. " +\
-            "I couldn't obtain an app id. " +\
-            "Please send an angry email to %s." % backlift_email
-        return
+    else:
+        fail(ERR_BADID)
 
 
 def init(path):
@@ -226,35 +276,46 @@ def push(path, skip_hidden):
 
     # scan path for files to upload 
 
-    files, count, configfile = collectFiles(path, skip_hidden)
-    if count > maxfiles:
-        print >> sys.stderr, "Too many files. Total: %d,  max: %d" % \
-            (count, maxfiles)
-        return
+    files, configfile = collectFiles(path, skip_hidden)
 
-    id = ''
-
-    # try to obtain app id from configfile, or create one
-
-    # import pdb; pdb.set_trace()
-
-    try:
-        handle = open(configfile, 'r')
-        cfg_str = handle.read()
-        cfg_data = yaml.safe_load(cfg_str)
-        handle.close()
-        id = cfg_data['_app_id']
-    except Exception as e:
-        pass
-
-    if not id:
-        print >> sys.stderr, "Ooops! Something's wrong. I couldn't obtain\n"+\
-            "an app id. To use the push command, you need an existing\n"+\
-            "backlift app created with either the startapp or init commands. "
-        return
+    id = get_id(configfile)
 
     upload_files(id, files)
 
+    return id
+
+
+def watch(path, skip_hidden):
+    # import pdb; pdb.set_trace()
+
+    id = push(path, skip_hidden)
+
+    before, configfile = scanFiles(path, skip_hidden)
+
+    try:
+        while 1:
+            time.sleep (0.5)
+            after, configfile = scanFiles(path, skip_hidden)
+            
+            added = [f for f in after if not f in before]
+            removed = [f for f in before if not f in after]
+            modified = [f for f in after if after[f] != before[f]]
+            
+            if added: print "Added: ", ", ".join (added)
+            if removed: print "Removed: ", ", ".join (removed)
+            if modified: print "Modified: ", ", ".join (modified)
+            
+            if added or removed or modified:    
+                files, configfile = collectFiles(path, skip_hidden)        
+                upload_files(id, files)
+    
+            before = after
+
+    except KeyboardInterrupt:
+        pass
+
+
+
 
 ## MAIN -----------------------------------------------------------------------
 
@@ -269,9 +330,11 @@ def execute_command_line():
     usagetxt = '%prog COMMAND [options]\n\n' +\
         'Commands:\n' +\
         '  register KEY\t\tRegister this computer for early access\n' +\
-        '  create NAME\t\tCreate a new backlift app\n' +\
+        '  create NAME\t\tCreate a new basic backlift app\n' +\
+        '  create:TYPE NAME\t\tCreate a new backlift app using the TYPE template\n' +\
         '  init\t\t\tInitialize backlift for an existing app\n' +\
-        '  push\t\t\tPush files up to backlift'
+        '  push\t\t\tPush files up to backlift\n' +\
+        '  watch\t\t\tObserve path and push files to backlift whenever they change'
 
     parser = optparse.OptionParser(usagetxt, version='%prog cli '+VERSION)
 
@@ -300,15 +363,23 @@ def execute_command_line():
                 parser.print_help()
             else:
                 register(args[1])
-        elif args[0].lower() == 'create':
+        elif args[0].lower().startswith('create'):
             if len(args) != 2:
                 parser.print_help()
             else:
-                create(options.path, args[1], 'basic')
+                tokens = args[0].lower().split(':')
+                if len(tokens) > 1:
+                    create(options.path, args[1], tokens[1])
+                else:
+                    create(options.path, args[1], 'basic')
         elif args[0].lower() == 'init':
             init(options.path)
         elif args[0].lower() == 'push':
             push(options.path, options.skip_hidden)
+        elif args[0].lower() == 'watch':
+            watch(options.path, options.skip_hidden)
+        else:
+            parser.print_help()
 
 if __name__ == "__main__":
     execute_command_line()
